import { StandardRetryToken } from "@smithy/types";
/**
 * @internal
 */
export declare const createDefaultRetryToken: ({ retryDelay, retryCount, retryCost, }: {
    retryDelay: number;
    retryCount: number;
    retryCost?: number | undefined;
}) => StandardRetryToken;
